%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                  samsara_dvr.m
%                   Russell Luke
%  Department of MathSci, University of Delaware  
%                        and 
%  Inst. f. Num. u. Angew. Math., Universitaet Goettingen
%                    JUNE. 2, 2010  
%
%  This work was supported by NSF-DMS grant 0712796. 
%
%
% DESCRIPTION:  driver for running the SAMSARA reverse communication 
%               optimization toolbox in non-reverse communication
%               mode, i.e. the user passes this routine the
%               function to be minimized, together with a
%               representation of the gradient of this function and
%               an initial point and the routine returns the
%               optimal value and optimal solution.  SAMSARA is not
%               by any means the most efficient toolbox for
%               these kind of ``standard" optimization problems. 
%               the power of SAMSARA is it's reverse communication
%               capability for very complex and large-scale objectives.
%
% INPUT:  objective function (which also computes the gradient), initial point, 
%             tolerances data structure (optional).
%
% OUTPUT:
%       xopt_vec, the optimal solution
%       fopt, optimal value
%
% USAGE: 
%        [xopt_vec,fopt] = samsara_dvr(objective_function,
%                                   x0_vec, [TOL])  
%    [...]  means an optional argument.  If TOL is input, it must be 
%    a data structure containing the folling fields:  TOL.ngrad,
%    TOL.step, TOL.Maxit.  Not all of these fields need be
%    nonempty, but they must be declared.  For example, one might
%    wish only to specify the Maxit count;  the coresponding TOL
%    structure would be TOL.ngrad=[]; TOL.step=[]; TOL.Maxit=100;
% 
% SUBROUTINE CALLS:
%      SAMSARA
%      Objective, 
%

function  [xopt_vec,fopt] = samsara_dvr(Objective, x0_vec, varargin)  
% [ngrad_TOL, step_TOL]
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initialization
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
dim           = length(x0_vec);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Tolerances
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if (length(varargin)==1)
  if (isempty(TOL.ngrad))
    ngrad_TOL = 1e-6;
  else 
    ngrad_TOL = TOL.ngrad;
  end
  if (isempty(TOL.step))
    step_TOL = 1e-12;
  else
    step_TOL = TOL.step;
  end
  if (isempty(TOL.Maxit))
    Maxit = 500;
  else
    Maxit = TOL.Maxit;
  end
else
    ngrad_TOL = 1e-6;
    step_TOL = 1e-12;
    Maxit = 500;
end 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Clear old data
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
!rm *.dat

%%%%%%%%%%%%%%%%%
% Main loop
%%%%%%%%%%%%%%%%%
iter=1;
xold_vec = x0_vec;
% SAMSARA needs two points:  the current point and a proposed point.  
% On the very first iteration the user can either generate the second 
% proposed point manually, or allow SAMSARA to automatically generate 
% the second point.  SAMSARA generates the second point by taking a scaling 
% of the steepest descent direction.  After the first iteration the current step is
% stored and updated in a file maintained by SAMSARA.  
[fold,gradfold_vec]=feval(Objective,xold_vec);
% fold=[];
% d_vec=-.1*gradfold_vec;
% xnew_vec=xold_vec+d_vec;
% stepsize=feval('norm',d_vec)
% [fnew,gradfnew_vec]=feval(Objective,xnew_vec);
% ngradfnew = sqrt(gradfnew_vec'*gradfnew_vec);
d_vec=[];
xnew_vec=[];
stepsize=999;
fnew=[];
gradfnew_vec = [];
ngradfnew=999;
while ((iter<Maxit)&&(ngradfnew>ngrad_TOL)&&(stepsize>step_TOL))
    [xnew_vec,xold_vec,fold,gradfold_vec]=...
        feval('samsara', xold_vec, xnew_vec, fold, fnew, gradfold_vec, gradfnew_vec);
    iter=iter+1;
    % new function evaluation:
    [fnew,gradfnew_vec] = feval(Objective,xnew_vec);
    ngradfnew = sqrt(gradfnew_vec'*gradfnew_vec);
end
if(stepsize<=step_TOL)
    disp('Algorithm stagnated:  stepsize tolerance violated.')
end
%%%%%%%%%%%%%%%%%
% end main loop
%%%%%%%%%%%%%%%%%
xopt_vec=xnew_vec;
fopt=fnew

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% prepare diagnostic data and return
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[fid,message]=fopen('state.dat','r');
if (fid<0)
    disp(message)
else
    state_vec=fread(fid,[1,inf],'real*8');
    fclose(fid);
end
Maxit=state_vec(end);
f_vec = state_vec(1:Maxit+1);
ngradf_vec = state_vec(Maxit+2:2*(Maxit+1));
stepsize_vec = state_vec(2*Maxit+3:3*(Maxit+1));
mu_vec = state_vec(3*Maxit+4:4*(Maxit+1));
iter = state_vec(4*(Maxit+1)+1)
fcounter = state_vec(4*(Maxit+1)+10)
trstep = state_vec(4*(Maxit+1)+11);




