from matlab import double
import matlab.engine
from numpy import abs, amax, array
from numpy.random import choice, randint, random

from problems.JWST.fft import FFT, IFFT
from problems.JWST.rl import RL_gradRL
from problems.JWST.util import Resize, data_processor


eng = matlab.engine.start_matlab()
eng.addpath('../../../matlab/test_problems/JWST')

EPS = 1e-8
ITERATIONS = 10


def gen_FFT_params():
    dims = randint(1, 3)
    dim = choice([16, 32, 64, 128, 256, 512, 1024], dims)
    return 1000.*random(dim).flatten()


def gen_Resize_params():
    M, N = (128, 128)
    Mnew = choice([32, 64, 128, 256, 512])
    return random((M, N)), int(Mnew), int(Mnew)


def gen_data_processor_params():
    return int(choice([32, 64, 128, 256, 512]))


def gen_RL_gradRL_params():
    res = 64
    x = 1e-12*random(2*res**2)
    params = data_processor(res, False)
    params['dim'] = array([res, res, params['diversity']])

    return res, x, params


def test_FFT():
    for _ in range(ITERATIONS):
        f_py = gen_FFT_params()
        f_mat = double(f_py.tolist())

        res_py = FFT(f_py)
        res_mat = eng.FFT(f_mat)
        diff = array(res_mat).flatten() - res_py.flatten()
        assert amax(abs(diff)) < EPS*amax(abs(array(res_mat)).flatten())


def test_IFFT():
    for _ in range(ITERATIONS):
        F_py = gen_FFT_params()
        F_mat = double(F_py.tolist())

        res_py = IFFT(F_py)
        res_mat = eng.IFFT(F_mat)
        diff = array(res_mat).flatten() - res_py.flatten()
        assert amax(abs(diff)) < EPS*amax(abs(array(res_mat)).flatten())


def test_Resize():
    for _ in range(ITERATIONS):
        A_py, Mnew, Nnew = gen_Resize_params()
        A_mat = double(A_py.tolist())

        res_mat = eng.Resize(A_mat, Mnew, Nnew)
        res_py = Resize(A_py, Mnew, Nnew)
        diff = array(res_mat).flatten() - res_py.flatten()
        assert amax(abs(diff)) < EPS*amax(abs(array(res_mat).flatten()))


def test_data_processor():
    for _ in range(ITERATIONS):
        newres = gen_data_processor_params()

        eng.cd('/home/benedikt/Projects/samsara/matlab/test_problems/JWST')
        res_mat = eng.data_processor(newres, 0)
        res_py = data_processor(newres, False)
        for key in ['k', 'aberration', 'Xi_A']:
            diff = array(res_mat[key]).flatten() - res_py[key].flatten()
            assert amax(abs(diff)) < EPS*amax(abs(array(res_mat[key]).flatten()))

        assert res_mat['diversity'] == res_py['diversity']


def test_RL_gradRL():
    for _ in range(ITERATIONS):
        res, x_py, params = gen_RL_gradRL_params()
        x_mat = eng.transpose(double(x_py.tolist()))
        res = float(res)

        eng.cd('/home/benedikt/Projects/samsara/matlab/test_problems/JWST')
        RL_mat, gradRL_mat = eng.test_RL_gradRL(res, x_mat,
                                                nargout=2)
        RL_py, gradRL_py = RL_gradRL(params, x_py)
        RL_diff = array(RL_mat).flatten() - RL_py.flatten()
        gradRL_diff = array(gradRL_mat).flatten() - gradRL_py.flatten()
        assert amax(abs(RL_diff)) <= EPS*amax(abs(array(RL_mat).flatten()))
        assert amax(abs(gradRL_diff)) < EPS*amax(abs(array(gradRL_mat).flatten()))
